@@ -25,18 +25,9 @@
 extern zend_module_entry mongodb_module_entry;
 
 /* FIXME: Its annoying to bump version. Move into phongo_version.h.in */
-#define PHP_MONGODB_VERSION "1.2.11"
+#define PHP_MONGODB_VERSION "1.3.0"
 #define PHP_MONGODB_STABILITY "stable"
 
-#ifdef PHP_WIN32
-#	define PHONGO_API __declspec(dllexport)
-#	include "ext/standard/flock_compat.h"
-#elif defined(__GNUC__) && __GNUC__ >= 4
-#	define PHONGO_API __attribute__ ((visibility("default")))
-#else
-#	define PHONGO_API
-#endif
-
 /* Structure for persisted libmongoc clients. The PID is included to ensure that
  * processes do not destroy clients created by other processes (relevant for
  * forking). We avoid using pid_t for Windows compatibility. */
@@ -50,6 +41,7 @@ ZEND_BEGIN_MODULE_GLOBALS(mongodb)
 	FILE *debug_fd;
 	bson_mem_vtable_t bsonMemVTable;
 	HashTable pclients;
+	HashTable *subscribers;
 ZEND_END_MODULE_GLOBALS(mongodb)
 
 #if PHP_VERSION_ID >= 70000
@@ -121,14 +113,14 @@ void phongo_throw_exception_from_bson_error_t(bson_error_t *error TSRMLS_DC);
 
 zend_object_handlers *phongo_get_std_object_handlers(void);
 
-void                     phongo_server_init          (zval *return_value, zval *manager, int server_id TSRMLS_DC);
+void                     phongo_server_init          (zval *return_value, mongoc_client_t *client, int server_id TSRMLS_DC);
 void                     phongo_readconcern_init     (zval *return_value, const mongoc_read_concern_t *read_concern TSRMLS_DC);
 void                     phongo_readpreference_init  (zval *return_value, const mongoc_read_prefs_t *read_prefs TSRMLS_DC);
 void                     phongo_writeconcern_init    (zval *return_value, const mongoc_write_concern_t *write_concern TSRMLS_DC);
 mongoc_bulk_operation_t* phongo_bulkwrite_init       (zend_bool ordered);
-bool                     phongo_execute_write        (zval *manager, const char *namespace, php_phongo_bulkwrite_t  *bulk_write, const mongoc_write_concern_t *write_concern, int server_id, zval *return_value, int return_value_used TSRMLS_DC);
-int                      phongo_execute_command      (zval *manager, const char *db, zval *zcommand, zval *zreadPreference, int server_id, zval *return_value, int return_value_used TSRMLS_DC);
-int                      phongo_execute_query        (zval *manager, const char *namespace, zval *zquery, zval *zreadPreference, int server_id, zval *return_value, int return_value_used TSRMLS_DC);
+bool                     phongo_execute_write        (mongoc_client_t *client, const char *namespace, php_phongo_bulkwrite_t  *bulk_write, const mongoc_write_concern_t *write_concern, int server_id, zval *return_value, int return_value_used TSRMLS_DC);
+int                      phongo_execute_command      (mongoc_client_t *client, const char *db, zval *zcommand, zval *zreadPreference, int server_id, zval *return_value, int return_value_used TSRMLS_DC);
+int                      phongo_execute_query        (mongoc_client_t *client, const char *namespace, zval *zquery, zval *zreadPreference, int server_id, zval *return_value, int return_value_used TSRMLS_DC);
 
 const mongoc_read_concern_t*  phongo_read_concern_from_zval   (zval *zread_concern TSRMLS_DC);
 const mongoc_read_prefs_t*    phongo_read_preference_from_zval(zval *zread_preference TSRMLS_DC);
@@ -146,6 +138,7 @@ void php_phongo_write_concern_to_zval(zval *retval, const mongoc_write_concern_t
 void php_phongo_cursor_to_zval(zval *retval, const mongoc_cursor_t *cursor);
 
 void phongo_manager_init(php_phongo_manager_t *manager, const char *uri_string, zval *options, zval *driverOptions TSRMLS_DC);
+int php_phongo_set_monitoring_callbacks(mongoc_client_t *client);
 void php_phongo_objectid_new_from_oid(zval *object, const bson_oid_t *oid TSRMLS_DC);
 void php_phongo_cursor_id_new_from_id(zval *object, int64_t cursorid TSRMLS_DC);
 void php_phongo_new_utcdatetime_from_epoch(zval *object, int64_t msec_since_epoch TSRMLS_DC);
@@ -174,13 +167,19 @@ zend_bool phongo_writeconcernerror_init(zval *return_value, bson_t *bson TSRMLS_
     ce->unserialize = zend_class_unserialize_deny; \
 } while(0);
 
-#ifdef PHP_DEBUG
-void _phongo_debug_bson(bson_t *bson);
-#else
-	#define _phongo_debug_bson(bson)
-#endif
-
-PHP_MINIT_FUNCTION(bson);
+#define PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, size) do { \
+    if (is_debug) {                                                             \
+        ALLOC_HASHTABLE(props);                                                 \
+        zend_hash_init((props), (size), NULL, ZVAL_PTR_DTOR, 0);                \
+    } else if ((intern)->properties) {                                          \
+        zend_hash_clean((intern)->properties);                                  \
+        (props) = (intern)->properties;                                         \
+    } else {                                                                    \
+        ALLOC_HASHTABLE(props);                                                 \
+        zend_hash_init((props), (size), NULL, ZVAL_PTR_DTOR, 0);                \
+        (intern)->properties = (props);                                         \
+    }                                                                           \
+} while(0);
 
 #endif /* PHONGO_H */
 
